/*
 * gsampler-option.c
 * Copyright (C) Lilian Monchalin 2011 <isaya07@users.sf.net>
 * 
 * gsampler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * gsampler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib/gstdio.h>
#include <glib/gi18n-lib.h>

#include "gsampler-option.h"

G_DEFINE_TYPE (GsamplerOption, gsampler_option, G_TYPE_OBJECT);

static GsamplerOption* option_instance = NULL;

#define GSAMPLER_OPTION_CONFIG_FILE "config.ini"

static gchar* 
gsampler_option_get_config_dir (GsamplerOption* self) 
{
	gchar* path = NULL;
	gboolean result;
	g_return_val_if_fail (self != NULL, NULL);
	
	path = g_build_filename (g_get_user_config_dir (), PACKAGE_NAME, NULL);
	result = g_file_test (path, G_FILE_TEST_EXISTS);
	if (!result) {
		g_mkdir (path, 0700);
	}
	return path;
}

static gchar* 
gsampler_option_create_file (GsamplerOption* self, 
                             const gchar* path, const gchar* contents) 
{
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (path != NULL, NULL);
	g_return_val_if_fail (contents != NULL, NULL);
																						 
	if (!g_file_test (path, G_FILE_TEST_EXISTS)) {
		self->priv->first_start = TRUE;
		FILE* stream;
		stream = fopen (path, "w");
		if (stream != NULL) {
			fprintf (stream, "%s", contents);
			fclose (stream);
		} else {
			g_warning (_("Can't create %s"), path);
		}
	}
	return g_strdup (path);
}

static gchar* 
gsampler_option_create_preferences_file (GsamplerOption* self, 
                                         const gchar* filename, const gchar* contents) 
{
	gchar* result = NULL;
	gchar* path;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (filename != NULL, NULL);
	g_return_val_if_fail (contents != NULL, NULL);
																						
	path = g_build_filename (gsampler_option_get_config_dir (self), filename, NULL);
	result = gsampler_option_create_file (self, path, contents);
	g_free (path);
	return result;
}

static gchar* 
gsampler_option_get_config_file (GsamplerOption* self) 
{
	gchar* result = NULL;
	g_return_val_if_fail (self != NULL, NULL);
	
	result = gsampler_option_create_preferences_file (self, GSAMPLER_OPTION_CONFIG_FILE, "[General]\n" \
"width=1024\n" \
"height=800\n" \
"paned1=250\n" \
"paned2=550\n" \
"font=Sans 8\n" \
"toolbar=true\n" \
"statusbar=true\n" \
"keyboard=true\n" \
"console=false\n" \
"tab=0\n" \
"\n" \
"[Serveur]\n" \
"serveur-autostart=true\n" \
"serveur-wait=1\n" \
"serveur-com=/usr/bin/linuxsampler\n" \
"serveur-port=8888\n" \
"serveur-host=0.0.0.0\n" \
"serveur-timeout=200\n" \
"db-path=\n" \
"\n" \
"[Client]\n" \
"default-audio=JACK\n" \
"default-midi=ALSA\n" \
"default-engine=GIG\n" \
"sample-path=\n" \
"script-path=");
	return result;
}

static GObject * 
gsampler_option_constructor (GType type, guint n_construct_properties, 
                             GObjectConstructParam * construct_properties) 
{
	GObject * obj;
	GsamplerOption* self;
	gchar* config = NULL;
	GError * error = NULL;
									 
	obj = G_OBJECT_CLASS (gsampler_option_parent_class)->constructor (type, n_construct_properties, construct_properties);
	self = GSAMPLER_OPTION (obj);
	self->priv->key_file = g_key_file_new ();
	config = gsampler_option_get_config_file (self);
	g_key_file_load_from_file (self->priv->key_file, config, G_KEY_FILE_NONE, &error);
	if (error != NULL) 
	{
		g_warning (_("Can't read config.ini file: %s"), error->message);
		g_error_free (error);
	}
	g_free (config);
	return obj;
}

static void 
gsampler_option_init (GsamplerOption * self) 
{
	self->priv = GSAMPLER_OPTION_GET_PRIVATE (self);
	self->priv->first_start = FALSE;
}

static void 
gsampler_option_finalize (GObject* obj) 
{
	GsamplerOption * self;
	self = GSAMPLER_OPTION (obj);
	g_key_file_free (self->priv->key_file);
}

static void 
gsampler_option_class_init (GsamplerOptionClass * klass) 
{
	gsampler_option_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GsamplerOptionPrivate));
	G_OBJECT_CLASS (klass)->finalize = gsampler_option_finalize;
	G_OBJECT_CLASS (klass)->constructor = gsampler_option_constructor;
}

GsamplerOption* 
gsampler_option_new (void) 
{
	return g_object_new (GSAMPLER_TYPE_OPTION, NULL);
}

GsamplerOption* 
gsampler_option_get_instance (void) 
{
	if (option_instance == NULL) 
	{
		option_instance = gsampler_option_new ();
	}
	return g_object_ref (option_instance);
}

gint 
gsampler_option_get_integer (GsamplerOption* self, const gchar* group, const gchar* key) 
{
	gint ret = 0;
	GError * error = NULL;
	g_return_val_if_fail (self != NULL, 0);
	g_return_val_if_fail (group != NULL, 0);
	g_return_val_if_fail (key != NULL, 0);
	
	ret = g_key_file_get_integer (self->priv->key_file, group, key, &error);
	if (error != NULL) 
	{
		g_warning ("file %s: line %d:  error: %s ", __FILE__, __LINE__, error->message);
		g_error_free (error);
		return -1;
	}
	return ret;
}

gchar* 
gsampler_option_get_string (GsamplerOption* self, const gchar* group, const gchar* key) 
{
	gchar* result = NULL;
	GError * error = NULL;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (group != NULL, NULL);
	g_return_val_if_fail (key != NULL, NULL);
	
	result = g_key_file_get_string (self->priv->key_file, group, key, &error);
	if (error != NULL) 
	{
		g_warning ("file %s: line %d:  error: %s ", __FILE__, __LINE__, error->message);
		g_error_free (error);
		g_free (result);
		return NULL;
	}
	return result;
}

gboolean 
gsampler_option_get_boolean (GsamplerOption* self, const gchar* group, const gchar* key) 
{
	gboolean result = FALSE;
	GError * error = NULL;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (group != NULL, FALSE);
	g_return_val_if_fail (key != NULL, FALSE);

	result = g_key_file_get_boolean (self->priv->key_file, group, key, &error);
	if (error != NULL) {
		g_warning ("file %s: line %d:  error: %s ", __FILE__, __LINE__, error->message);
		g_error_free (error);
		return FALSE;
	}
	return result;
}

void 
gsampler_option_set_integer (GsamplerOption* self, const gchar* group, 
                                  const gchar* key, gint val) 
{
	g_return_if_fail (self != NULL);
	g_return_if_fail (group != NULL);
	g_return_if_fail (key != NULL);
	
	g_key_file_set_integer (self->priv->key_file, group, key, val);
}

void 
gsampler_option_set_string (GsamplerOption* self, const gchar* group, 
                                 const gchar* key, const gchar* val) 
{
	g_return_if_fail (self != NULL);
	g_return_if_fail (group != NULL);
	g_return_if_fail (key != NULL);
	g_return_if_fail (val != NULL);
	
	g_key_file_set_string (self->priv->key_file, group, key, val);
}

void 
gsampler_option_set_boolean (GsamplerOption* self, const gchar* group, 
                                  const gchar* key, gboolean val) 
{
	g_return_if_fail (self != NULL);
	g_return_if_fail (group != NULL);
	g_return_if_fail (key != NULL);
																		
	g_key_file_set_boolean (self->priv->key_file, group, key, val);
}

void 
gsampler_option_save_keyfile (GsamplerOption* self) 
{
	gchar* file = NULL;
	gchar* new_content = NULL;
	GError * error = NULL;
	g_return_if_fail (self != NULL);
	
	file = gsampler_option_get_config_file (self);
	new_content = g_key_file_to_data (self->priv->key_file, NULL, NULL);
	g_file_set_contents (file, new_content, (-1), &error);
	g_free (file);
	g_free (new_content);
	if (error != NULL) 
	{
		g_warning ("file %s: line %d: error: %s ", __FILE__, __LINE__, error->message);
		g_error_free (error);
		return;
	}
}

gboolean
gsampler_option_is_first_start (GsamplerOption* self)
{
	return self->priv->first_start;
}